#!/usr/bin/env python
# -*- coding: UTF-8 -*-
'''
Created on 05.02.2010

@author: iarspider
'''

import ConfigParser
from xml.dom import minidom
from xmlnode import Node
import sys
import copy
import string

class ini2xml:
    cp = ConfigParser.ConfigParser()
    root = Node('')
    display = {}
    handlers = {}
    types = {0:'Null', 1:'Integer', 2:'String', 3:'Data', 4:'List', 5:'StrList', 6:'Icon',
             7:'Real', 8:'Color', 9:'Script', 10:'Stream', 11:'Bitmap', 12:'Wave',
             13:'Array', 14:'Enum', 15:'Font', 16:'Matr', 17:'Jpeg', 18:'Menu',
             19:'Code', 20:'LinkElement',  21:'Flags'}
    mtypes = {1:'Work', 2:'Event', 3:'Var', 4:'Data'}
    
    
    def my_get(self, sect, key):
        try:
            s = self.cp.get(sect, string.lower(key))
        except ConfigParser.NoOptionError:
            return False
        except:
            print "Unexpected error:", sys.exc_info()[0]
            raise
        else:
            return s

    def ProcessAbout(self):
        for item in self.cp.items('About'):
            if (item[0] == 'class'):
                continue
            if (item[0] == 'info'):
                info = Node('Info')
                info += Node('text', item[1], lang='ru')
                self.root += info
            else:
                self.root.setAttribute(string.lower(item[0]), item[1])

    def ProcessType(self):
        val = self.my_get('Type', 'Class')
        if (val):
            self.root = Node(val)
        else:
            sys.exit ("Type not specified!")

        for item in self.cp.items('Type'):
            if (item[0] == 'class'):
                continue
            if (item[0] == 'interfaces'):
                iface = Node('Interfaces')
                for item in item[1].split(','):
                    iface += Node('Interface', item)
                self.root += iface
                continue
            if (item[0] == 'info'):
                info = Node('Info')
                info += Node('text', item[1], lang='ru')
                self.root += info
            else:
                self.root.setAttribute(string.lower(item[0]), item[1])  
        

    def ProcessEdit(self):
        val = self.my_get('Edit', 'Class')
        if (not val):
            return

        self.root += Node('EditClass', val)        
        for item in self.cp.items('Edit'):
            if (item[0] == 'class'):
                continue
            else:
                self.display[item[0]] = [item[1], False]
        
    def ProcessHandlers(self):
        try:
            for item in self.cp.items('Handlers'):
                for prop in item[1].split(','):
                    self.handlers[prop] = [item[0], False]
        except ConfigParser.NoSectionError:
            return

    def ProcessProps(self):
        props = Node('Properties')
        localroot = None
        try:
            for item in self.cp.items('Property'):
                name = item[0].strip('+@')
                if (name.startswith('##')):
                    if (name != '##'):
                        localroot = Node('Group')
                        localroot.setAttribute('name', name[2:])
                        continue
                    else:
                        props += localroot
                        localroot = None
                        continue
                        
                prop = Node('Property')
                        
                prop.setAttribute('Name', name)
                if (item[0].startswith('+')):
                    prop.setAttribute('edit', 'yes')
                if (item[0].startswith('@')):
                    prop.setAttribute('work', 'yes')
                if (name in self.handlers):
                    prop.setAttribute('handler', self.handlers[name][0])
                    self.handlers[name][1] = True
                if (name is self.display):
                    prop.setAttribute('display', self.display[name][0])
                    self.display[name][1] = True
                x = item[1].replace('||', '$$').split('|')

                if (len(x[0].strip()) > 0):
                    hint = Node('Hint')
                    hint += Node('text', x[0].replace('$$', '||'), lang='ru')
                    prop += hint

                vtype = x[1]
                if (x[1].isdigit()):
                    prop.setAttribute('type', self.types[int(vtype)])
                
                try:
                    default = x[2]
                except:
                    default = False

                if (default != False):
                    prop.setAttribute('default', default)

                try:
                    vlist = x[3]
                except:
                    vlist = False
                        
                if (vlist != False):
                    l = Node('List')
                    for vak in vlist.split(','):
                        l += Node('Item', vak)
                    prop += l
                 
                if (localroot != None):
                    localroot += prop
                else:
                    props += prop
            #Push overloaded 'handler's and 'display's
            for nom, item in self.display.iteritems():
                if (not item[1]):
                    prop = Node('Property', display = item[0], name = nom)
                    self.display[nom][1] = True
                    if (nom in self.handlers):
                        prop.setAttribute('handler', self.handlers[nom][0])
                        self.handlers[nom][1] = True
                    props += prop
                    
            for nom, item in self.handlers.iteritems():
                if (not item[1]):
                    prop = Node('Property', handler = item[0], name = nom)
                    self.handlers[nom][1] = True
                    if (nom in self.display):
                        prop.setAttribute('handler', self.display[nom][0])
                        self.display[nom][1] = True
                    props += prop
                    
            self.root += props                                
        except ConfigParser.NoSectionError:
            print "No properties"
            return

    def ProcessMethods(self):
        methods = Node('Methods')
        try:
            for item in self.cp.items('Methods'):
                name = item[0]
                hint,etype,vtype = item[1].replace('||', '$$').split('|')
                method = Node(self.mtypes[int(etype)])
                method.setAttribute('name', name.strip('*'))
                if (name.startswith('*')):
                    method.setAttribute('hidden', 'yes')

                if (len(vtype) == 0):
                    vtype = '3'
                    
                method.setAttribute('return', self.types[int(vtype)])
                hi = Node('Hint')
                hi += Node('text', hint, lang='ru')
                method += hi
                methods += method

            self.root += methods
                
        except ConfigParser.NoSectionError:
            print "No methods"
            return
        

    def ConvertConfig(self, file):
        self.cp.read(file)

        self.ProcessType()
        self.ProcessAbout()
        self.ProcessEdit()
        self.ProcessHandlers()
        self.ProcessProps()
        self.ProcessMethods()

    #    for sec in cp.sections():
    #        name = string.lower(sec)
    #        print(name)
    #        for opt in cp.options(sec):
    #            print "\t"+string.strip(opt)
        DOMimpl = minidom.getDOMImplementation()

        #Create Document
        xmldoc = DOMimpl.createDocument(None, None, None)
        xmldoc.appendChild(self.root.dom())

        return xmldoc

if __name__== "__main__":
    parser = ini2xml()
    x = parser.ConvertConfig("StringTableMT.ini")

    f = open("StringTableMT.xml", "w")
    x.writexml(f, "  ", "  ", "\n")
    f.close()
